
# coding=utf-8
'''
Created on 11.4.2013

'''
__author__ = "Jarkko Aalto \n Timo Konu \n Samuli Kärkkäinen \n Samuli Rahkonen \n Miika Raunio"
__versio__ = "1.0"

import logging
import os
from PyQt4 import QtGui
from PyQt4 import uic


class ProjectNewDialog(QtGui.QDialog):
    '''Dialog creating a new project.
    '''
    def __init__(self, parent):
        '''Inits energy spectrum dialog.
        
        Args:
            parent: Ibasoft class object.
        '''
        super().__init__()
        self.parent = parent
        self.folder = None  # Temporary for browsing folder
        self.directory = None
        
        self.ui = uic.loadUi(os.path.join("ui_files", "ui_new_project.ui"), self)
        self.default_directory_used = True
        self.ui.projectDirectoryLineEdit.setText(
                                     self.parent.settings.get_project_directory())
        
        # Connect buttons
        self.ui.pushOk.clicked.connect(self.__create_project) 
        self.ui.pushCancel.clicked.connect(self.close) 
        self.ui.browseFolderButton.clicked.connect(self.__browser_folder)
        self.ui.useDefaultCheckBox.clicked.connect(self.__change_directory)
        
        self.exec_()
    
    
    def __change_directory(self):
        self.default_directory_used = not self.default_directory_used 
        if self.default_directory_used:
            self.ui.projectDirectoryLineEdit.setText(
                                     self.parent.settings.get_project_directory())
    
    
    def __browser_folder(self):
        folder = QtGui.QFileDialog.getExistingDirectory(self,
                                     self.ui.browseFolderButton.text())
        if folder:
            self.folder = folder
            self.ui.projectDirectoryLineEdit.setText(folder)
        
        
    def __create_project(self):
        self.folder = self.ui.projectDirectoryLineEdit.text()
        self.name = self.ui.projectNameLineEdit.text()
        # TODO: check for valid folder needed
        # TODO: Get rid of print -> message window perhaps
        if not self.folder:  
            print("Project folder required!")
            return
        if not self.name: 
            print("Project name required!")
            return
        try:
            directory = os.path.join(self.folder, self.name)
            if not os.path.exists(directory):
                os.makedirs(directory)
                self.directory = directory
                logging.getLogger("project").info("Created the project.")
            else:
                print("Folder already exists: {0}".format(directory))
                return
            self.close()
        except:
            print("We've done something wrong. Most likely invalid project name.")
