# coding=utf-8
'''
Created on 12.4.2013
'''
__author__ = "Jarkko Aalto \n Timo Konu \n Samuli Kärkkäinen \n Samuli Rahkonen \n Miika Raunio"
__versio__ = "1.0"

import os
from Modules.MeasuringSettings import MeasuringSettings
from Modules.CalibrationParameters import CalibrationParameters
from Modules.DepthProfileSettings import DepthProfileSettings

class Settings:
    '''Settings class to handle settings of project and measurement.
    '''
    def __init__(self, directory=None, project_settings=None):
        '''Inits Settings class.
        '''
        self.project_settings = project_settings # TODO: iffit alempien settingsien sisään
        #if directory:  # Settings from a given directory override the other options
            
        #measuring_unit_settings_filepath = os.path.join(directory, self.measuring_unit_settings_filename)
        self.measuring_unit_settings = MeasuringSettings(directory)
        
        #self.calibration_settings_filename = "calibration_settings.ini"
        self.calibration_settings = CalibrationParameters(directory)
        
        #self.depth_profile_settings_filename = "depth_profile_settings.ini"
        self.depth_profile_settings = DepthProfileSettings(directory)
        #else:  # If there are given settings for the measurement, use them. Otherwise use empty settings.
        #    self.measuring_unit_settings = MeasuringSettings()
        #    self.calibration_settings = CalibrationParameters()
        #    self.depth_profile_settings = DepthProfileSettings()
    
    
    def get_measurement_settings(self):
        """Get the measurement specific settings.
        
        Get currently used settings by measurement. If measurement uses project
        settings (by default), it will return project's settings instead.
        
        Returns:
            Settings object that has all the references to settings that a 
            measurement uses.
        """
        if self.measuring_unit_settings.use_settings == "MEASUREMENT":
            use_measuring = self.measuring_unit_settings
        else:  # If there is "" or "PROJECT"
            use_measuring = self.project_settings.measuring_unit_settings
            
        if self.calibration_settings.use_settings == "MEASUREMENT":
            use_calibration = self.calibration_settings
        else:
            use_calibration = self.project_settings.calibration_settings    
            
        if self.depth_profile_settings.use_settings == "MEASUREMENT":
            use_depth = self.depth_profile_settings
        else:
            use_depth = self.project_settings.depth_profile_settings
            
        settings = Settings()
        settings.measuring_unit_settings = use_measuring # TODO: Tee metodit eikä viitata suoraan
        settings.calibration_settings = use_calibration
        settings.depth_profile_settings = use_depth
        return settings  
         
    
    
