# coding=utf-8
'''
Created on 18.4.2013

'''
__author__ = "Jarkko Aalto \n Timo Konu \n Samuli Kärkkäinen \n Samuli Rahkonen \n Miika Raunio"
__versio__ = "1.0"

from Widgets.MatplotlibWidget import MatplotlibWidget
from PyQt4 import QtGui

class MatplotlibCalibrationLinearFittingWidget(MatplotlibWidget):
    '''Energy spectrum widget
    '''
    def __init__(self, parent, tof_calibration, dialog=None, old_params=None):
        '''Inits Energy Spectrum widget.
        '''
        super().__init__(parent)
        super().fork_toolbar_buttons()
        self.dialog = dialog
        self.old_params = old_params

        self.tof_calibration = tof_calibration
        
        self.__max_x_value = 0  # Used to as the maximum x value for the plot
        self.enable_selection_tool = False
        self.on_draw()

    
    
    def __update_dialog_values(self):
        """Updates the parent dialog's fields with the calculated values.
        """
        slope = self.tof_calibration.slope
        offset = self.tof_calibration.offset
        
        
        if slope is not None and offset is not None:
            self.dialog.ui.slopeLineEdit.setText(str(slope))
            self.dialog.ui.offsetLineEdit.setText(str(offset))
        else:    
            self.dialog.ui.slopeLineEdit.setText("")
            self.dialog.ui.offsetLineEdit.setText("")
        
        
    
    def on_draw(self):
        self.axes.clear()  # Clear old stuff

        points = self.tof_calibration.get_points()  # Get the point set as a tuple(x, y). x = time of flight channel, y = time of flight seconds
        self.axes.plot(points[0], points[1], ".")
        
        x_min, x_max = self.axes.get_xlim() 
        y_min, y_max = self.axes.get_ylim()  
        
        #self.__update_dialog_values()
        #self.dialog.ui.slopeLineEdit.setText("")
        #self.dialog.ui.offsetLineEdit.setText("")
        
        if points[0]:
            for i in range(len(points[0])):
                self.axes.text(points[0][i],
                               points[1][i],
                               s=" {0}".format(points[2][i]),  # Element name (and isotope)
                               va="top",
                               ha="left")

        # Draw old calibration line
        if self.old_params:
            old_fit_points_x, old_fit_points_y = self.tof_calibration.get_linear_fit_points(self.old_params, x_min, x_max, 2)
            print("Old parameters: " + str(self.old_params) + " " + str(old_fit_points_x) + " " + str(old_fit_points_y))
            self.axes.plot(old_fit_points_x, old_fit_points_y, color="0.8", linestyle="--")
                
        
                
        params = self.tof_calibration.fit_linear_function(points[0], points[1], 0, 0)  # Returns the parameters as a tuple (a, b)
        print(str(params))
        print("X_MAX: {0}".format(x_max))
        print("Y_MIN: {0}".format(y_min))
        #if len(points[0]) > 1 and len(points[1]) > 1:    
        if params[0] is not None and points[1] is not None:
            fit_points_x, fit_points_y = self.tof_calibration.get_linear_fit_points(params, x_min, x_max, 2)
            self.axes.plot(fit_points_x, fit_points_y, "-")
            
           
            #self.dialog.ui.slopeLineEdit.setText(str(params[0]))
            #self.dialog.ui.offsetLineEdit.setText(str(params[1]))
        
        self.__update_dialog_values()
        if x_max > 0.09 and x_max < 1.01:  # This works... # TODO: tarvitaanko??
            x_max = self.axes.get_xlim()[1]
        if y_max > 0.09 and y_max < 1.01:
            y_max = self.axes.get_ylim()[1]
            
        # Set limits accordingly
        self.axes.set_ylim([y_min, y_max])
        self.axes.set_xlim([x_min, x_max])
        
        self.axes.set_ylabel("Time of Flight [s]")
        self.axes.set_xlabel("Channel Number [Ch]")
        
        # Remove axis ticks
        if not self.show_axis_ticks:
            for tick in self.axes.yaxis.get_major_ticks():
                tick.label1On = False
                tick.label2On = False
            for tick in self.axes.xaxis.get_major_ticks():
                tick.label1On = False
                tick.label2On = False

        # Draw magic
        self.canvas.draw()
        
        
    def enable_clicks(self):
        self.enable_selection_tool = not self.enable_selection_tool
        
