# coding=utf-8
'''
Created on 21.3.2013

'''
__author__ = "Jarkko Aalto \n Timo Konu \n Samuli Kärkkäinen \n Samuli Rahkonen \n Miika Raunio"
__versio__ = "1.0"

from Modules.Element import Element
from Widgets.MatplotlibWidget import MatplotlibWidget


class MatplotlibElementLossesWidget(MatplotlibWidget):
    '''Energy spectrum widget
    '''
    def __init__(self, parent, split, legend=True, y_scale=0):
        '''Inits Energy Spectrum widget.
        '''
        super().__init__(parent)
        super().fork_toolbar_buttons()
        self.draw_legend = legend
        self.split = split
        self.y_scale = y_scale
        self.selection_colors = parent.parent.measurement.selector.get_colors()
        self.on_draw()
        
    def on_draw(self):
        self.axes.clear()  # Clear old stuff
        
        keys = sorted(self.split.keys())
        for key in keys:
            cut_file = key.split('.')
            element_object = Element(cut_file[0].strip())
            element, isotope = element_object.get_element_and_isotope()
            color_string = "{0}{1}{2}".format(isotope, element, cut_file[1])
            if not color_string in self.selection_colors:
                color = "red"
            else:
                color = self.selection_colors[color_string]
            
            if len(cut_file) == 2:
                label = r"$^{" + str(isotope) + "}$" + element
            else: 
                label = r"$^{" + str(isotope) + "}$" + element + "$_{split: " + cut_file[2] + "}$"
            self.axes.plot(self.split[key],
                           color=color,
                           label=label)

        if self.draw_legend:
            box = self.axes.get_position()
            self.axes.set_position([box.x0, box.y0, box.width * 0.8, box.height])
            
            handles, labels = self.axes.get_legend_handles_labels()
            leg = self.axes.legend(handles, labels, loc=3, bbox_to_anchor=(1.05, 0))
            for handle in leg.legendHandles:
                handle.set_linewidth(3.0)
        
        # Scale based on values
        x_min, x_max = self.axes.get_xlim()
        y_min, y_max = self.axes.get_ylim()
        
        # Y-axis scaling option, 0 = 0-max, 1 = min-max
        if self.y_scale == 0:
            y_min = 0
        
        # Set limits accordingly
        self.axes.set_ylim([y_min, y_max]) 
        self.axes.set_xlim([x_min, x_max])

        self.axes.set_ylabel("Count")
        self.axes.set_xlabel("Split")
             
        # Remove axis ticks
        if not self.show_axis_ticks:
            for tick in self.axes.yaxis.get_major_ticks():
                tick.label1On = False
                tick.label2On = False
            for tick in self.axes.xaxis.get_major_ticks():
                tick.label1On = False
                tick.label2On = False
        self.canvas.draw()
        
